/*
 * Copyright (C) 2009-2011 Sébastien Villemot <sebastien.villemot@ens.fr>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _MULTI_COUNTRY_CGA_SOLUTION_HH
#define _MULTI_COUNTRY_CGA_SOLUTION_HH

#include <string>

#include <gsl/gsl_matrix.h>
#include <gsl/gsl_spline.h>
#include <gsl/gsl_roots.h>

#include "ModelSolution.hh"
#include "MultiCountryRBCA2.hh"

class MultiCountryCGASolution : public ModelSolution
{
private:
  // Variables common to all specs
  gsl_matrix *V_prime, *z0_prime;
  int VI_dim1;
  double *VI;
  gsl_vector *meanx, *sigz0, *stdx, *stdz;
  double v;
  int NC;

  // Temporary variables
  gsl_vector *xnorm, *znorm, *crbf, *dist_z_z0, *rbfn, *instrs, *ord2_vec;
  gsl_matrix *z1, *vi, *VI_tmp, *ord2_mat;

  double *VIL;
  int VIL_dim2;
  gsl_matrix *VIL_tmp;
  gsl_matrix *vil;

  struct my_params
  {
    const double *a_curr, *k_prev, *k_curr;
    const ModelSpec *modspec;
  };

protected:
  virtual void policy_func_internal(const gsl_vector *y_prev, const gsl_vector *shocks, gsl_vector *y_curr);
public:
  MultiCountryCGASolution(const MultiCountryRBCA2 &modspec_arg) throw (UnsupportedSpec);
  virtual ~MultiCountryCGASolution();
  virtual std::string name() const;
};

#endif // !_MULTI_COUNTRY_CGA_SOLUTION_HH
