/*
 * Copyright (C) 2011 Sébastien Villemot <sebastien.villemot@ens.fr>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _PER_SOLUTION_HH
#define _PER_SOLUTION_HH

#include <gsl/gsl_matrix.h>

#include "ModelSolution.hh"

class PerSolution : public ModelSolution
{
protected:
  //! Number of variables
  const int ny;
  //! Number of state variables (including shocks)
  const int nz;
  gsl_vector *ys, *g_0;
  gsl_matrix *g_1, *g_2, *g_3;
private:
  //! Temporary variables for policy function computation
  gsl_vector *state, *state2, *state3, *y_reordered;
protected:
  void policy_func_perturbation(const gsl_vector *y_prev, const gsl_vector *shocks, gsl_vector *y_curr);
  //! Create state vector in Dynare internal order, in deviation from steady state
  virtual void create_state_vector(const gsl_vector *y_prev, const gsl_vector *shocks, gsl_vector *state) const = 0;
  //! Move from Dynare internal order to the order used by the test bench
  virtual void reorder_from_dynare(const gsl_vector *y_dynare, gsl_vector *y_curr) const = 0;
public:
  //! Order of approximation
  const int order;
  PerSolution(const ModelSpec &modspec_arg, const std::string &matname, int order_arg) throw (UnsupportedSpec);
  virtual ~PerSolution();
};

#endif // !_PER_SOLUTION_HH
