/*
  Dynare++ source file describing the multi-country RBC model used in the JEDC
  comparison project.

  This file needs to be processed by the Dynare preprocessor before feeding it
  to Dynare++. The number of countries has to be set in the macro-variable "N",
  within a file called "config.mod".
*/

/*
 * Copyright (C) 2010-2011 Sébastien Villemot <sebastien.villemot@ens.fr>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

@#include "config.mod"

// The order of declaration matches that of the test bench
@#for j in 1:N
var c@{j};
@#endfor
@#for j in 1:N
var l@{j};
@#endfor
@#for j in 1:N
var i@{j};
@#endfor
@#for j in 1:N
var k@{j};
@#endfor
@#for j in 1:N
var a@{j};
@#endfor
var lambda;

@#for j in 1:N
varexo e@{j};
parameters tau@{j}, gamma@{j}, eta@{j}, b@{j};
@#endfor

varexo e;
parameters beta, phi, delta, rho, sigma, A, alpha;

beta = 0.99;
alpha = 0.36;
delta = 0.025;
sigma = 0.01;
rho = 0.95;
phi = 0.5;

A = (1-beta)/alpha/beta;

@#for j in 1:N
gamma@{j} = 0.25 + (@{j}-1)*(1-0.25)/(@{N}-1);
eta@{j} = 0.1 + (@{j}-1)*(1-0.1)/(@{N}-1);
tau@{j} = A^(1/gamma@{j});
b@{j} = (1-alpha)*A^(1-1/gamma@{j});
@#endfor

model;
@#for j in 1:N
tau@{j}*c@{j}^(-1/gamma@{j})=lambda;
tau@{j}*b@{j}*l@{j}^(1/eta@{j})=lambda*a@{j}*(1-alpha)*A*(k@{j}(-1)/l@{j})^alpha;
lambda*(1+phi*(i@{j}/k@{j}(-1)-delta))=beta*lambda(+1)*(1+a@{j}(+1)*alpha*A*(l@{j}(+1)/k@{j})^(1-alpha)+phi*(1-delta+i@{j}(+1)/k@{j}-0.5*(i@{j}(+1)/k@{j}-delta))*(i@{j}(+1)/k@{j}-delta));
k@{j}=(1-delta)*k@{j}(-1)+i@{j};
log(a@{j})=rho*log(a@{j}(-1))+sigma*(e@{j}+e);
@#endfor
@#for j in 1:N
+c@{j}+i@{j}-delta*k@{j}(-1)
@#endfor
=
@#for j in 1:N
+a@{j}*A*k@{j}(-1)^alpha*l@{j}^(1-alpha)-phi/2*k@{j}(-1)*(i@{j}/k@{j}(-1)-delta)^2
@#endfor
;
end;

initval;
@#for j in 1:N
l@{j} = 1;
k@{j} = 1;
a@{j} = 1;
i@{j} = delta;
c@{j} = A;
@#endfor
lambda = 1;
end;

vcov = [
@#for j in 1:(N+1)
@# define matline = ""
@# for l in 1:(N+1)
@#  if j == l
@#   define matline = matline + "1 "
@#  else
@#   define matline = matline + "0 "
@#  endif
@# endfor
@# define matline = matline + ";"
@{matline}
@#endfor
];
