### Creates tables 1 and 2 (maximum absolute errors by equation on a sphere and
### on a stochastic simulation)

### Does not fill the computation times, nor the accuracy frontier in Kollman
### et al. (2011)

source("interface.R")

### Number of countries (N=2 for table 1, N=8 for table 2)
N <- 2

radia <- c(0.01, 0.1, 0.3)

specs <- character()

sol.time <- list()
test1 <- list()
test2 <- list()

for (o in 1:3)
  for (m in c("per", "hyb"))
    {
      ms <- multi.country.solution(n = N, solution.method = m, order = o)
      s <- ms$sol

      spec <- paste(m, o, sep = "")
      specs <- c(specs, spec)

      sol.time[[spec]] <- ms$sol.time
      test1[[spec]] <- list()
      for (r in radia)
        test1[[spec]][[as.character(r)]] <- accuracy.test1(s, radius = r)
      test2[[spec]] <- accuracy.test2(s)
  }


## Compute maximum error over similar equations

compute.max.errs <- function(v, is.test2 = FALSE)
  {
    if (is.test2)
      i <- 6*N+2
    else
      i <- 7*N+1
    
    list(EulerEq = max(abs(v$data[(i+2*N+1):(i+3*N), ])),
         MUCons = max(abs(v$data[(i+1):(i+N), ])),
         MULabor = max(abs(v$data[(i+N+1):(i+2*N), ])),
         WorResConst = max(abs(v$data[(i+5*N+1), ])))
  }

errs.test1 <- list()
errs.test2 <- list()

for (s in specs)
  {
    errs.test1[[s]] <- list()
    for (r in radia)
      errs.test1[[s]][[as.character(r)]] <- compute.max.errs(test1[[s]][[as.character(r)]])
    errs.test2[[s]] <- compute.max.errs(test2[[s]], is.test2 = TRUE)
  }

## Output file

sink(file = paste("table-euler-errors-N", N, ".tex", sep = ""))

cat("\\documentclass{article}", fill = TRUE)
cat("\\begin{document}", fill = TRUE)

cat("\\begin{tabular}{l|rr|rr|rr|r}", fill = TRUE)
cat("\\hline\\hline", fill = TRUE)
cat("& \\multicolumn{2}{|c|}{1st order} & \\multicolumn{2}{|c|}{2nd order} & \\multicolumn{2}{|c}{3rd order} & Accuracy frontier \\\\", fill = TRUE)
cat("& PER & HYB & PER & HYB & PER & HYB & in KMMP (2011) \\\\ \\hline \\hline", fill = TRUE)
cat("Solution time &  &  &  &  &  &  & \\\\ \\hline\\hline", fill = TRUE)
cat("\\multicolumn{8}{l}{} \\\\", fill = TRUE)

my.format <- function(x, meth = NA)
  {
    if (x <= 1e-10)
      "-"
    else
      {
        r <- format(log10(x), digits = 3, nsmall = 2)
        if (!is.na(meth))
          r <- paste(r, " (", meth, ")", sep = "")
        r
      }
  }

for (r in radia)
  {
    cat("\\multicolumn{8}{l}{Radius $r=", r, "$} \\\\ \\hline", fill = TRUE)

    ## Print results
    for (e in names(errs.test1[[1]][[as.character(r)]]))
      {
        cat(e)
        for (s in specs)
          {
            cat(" & ", my.format(errs.test1[[s]][[as.character(r)]][[e]]))
          }
        cat(" & \\\\", fill = TRUE)
      }

    cat("\\hline", fill = TRUE)
    
    cat("\\textit{Overall} ")
    for (s in specs)
      {
        cat(" & ", my.format(max(as.numeric(errs.test1[[s]][[as.character(r)]]))))
      }
    cat(" & \\\\", fill = TRUE)
    
    cat("\\hline", fill = TRUE)

    cat("Test time")
    for (s in specs)
      cat(" & ", format(test1[[s]][[as.character(r)]]$test.time, nsmall = 2))
    cat("  & \\\\ \\hline", fill = TRUE)

    cat("\\multicolumn{8}{l}{} \\\\", fill = TRUE)

  }

cat("\\multicolumn{8}{l}{Stochastic simulation} \\\\ \\hline", fill = TRUE)

for (e in names(errs.test2[[1]]))
  {
    cat(e)
    for (s in specs)
      {
        cat(" & ", my.format(errs.test2[[s]][[e]]))
      }
    cat(" & \\\\", fill = TRUE)
  }

cat("\\hline", fill = TRUE)

cat("\\textit{Overall} ")
for (s in specs)
  {
    cat(" & ", my.format(max(as.numeric(errs.test2[[s]]))))
  }
cat(" & \\\\", fill = TRUE)

cat("\\hline", fill = TRUE)

cat("Test time")
for (s in specs)
  cat(" & ", format(test2[[s]]$test.time, nsmall = 2))
cat("  & \\\\ \\hline", fill = TRUE)

cat("\\multicolumn{8}{l}{} \\\\", fill = TRUE)


cat("\\end{tabular}", fill = TRUE)

cat("\\end{document}", fill = TRUE)

sink()
